#coding:utf-8
import ConfigParser
import os
import time
def updateSelf(config,configPath):
    try:
        config.write(open(configPath,'w'))
    except:
        print '目录%s下的配置文件更新失败！！！'%(configPath)
class config():
    exist=False
    def __init__(self):
        self.config=ConfigParser.ConfigParser()
        s='' #s是配置文件初始化所在路径，和configPath是一样的
        for item in self.configPath.split('/'):
            s=s+item+'/'
            if not os.path.exists(s):
                os.mkdir(s)
        if( not os.path.exists(s+'/'+self.configName) )or (len(open(s+'/'+self.configName).read())==0):
            print 'building a new config file ...'
            with open (s+'/'+self.configName,'w') as fp:
                fp.write(self.content)
                print 'building is finished'
                print 'the config file %s is in %s'%(self.configName,s)
        self.config.read(s+'/'+self.configName)
    def getConfig(self,section,key):
        return self.config.get(section,key)
    def setConfig(self,section,key,value):
        self.config.set(section,key,value)
        updateSelf(self.config,self.configPath+'/'+self.configName)
    def addSection(self,section):
        self.config.add_section(section)
        updateSelf(self.config,self.configPath+'/'+self.configName)
    def items(self,section):
        return self.config.items(section)
    def deleteSection(self,section):
        self.config.remove_section(section)
        updateSelf(self.config,self.configPath+'/'+self.configName)
    def deleteOption(self,section,key):
        self.config.remove_option(section,key)
        updateSelf(self.config,self.configPath+'/'+self.configName)
    def isExistOption(self,section,key):
        return self.config.has_option(section,key)

class systemConfig(config):
    path=os.path.split(os.path.abspath(__file__))[0]
    configPath=os.path.split(os.path.split(path)[0])[0]
    configName='system'+'.conf'
    content='''
[check]
[system]
daynum=%s
%s=%s
series=%s
headScript = <script src='https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.2/MathJax.js?config=TeX-MML-AM_CHTML'></script>
[optional]
series=daily
theme=m1
indexTheme=index1
[theme]
'''%(time.strftime('%d'),
        time.strftime('%d'),
        os.path.split(os.path.split(os.path.split(os.path.abspath(__file__))[0])[0])[0]+'/bin'+'/md'+'/'+time.strftime('%Y')+'/'+time.strftime('%m')+'/'+time.strftime('%d'),
        os.path.split(os.path.split(os.path.split(os.path.abspath(__file__))[0])[0])[0]+'/bin'+'/series')





class userConfig(config):
    path=os.path.split(os.path.abspath(__file__))[0]
    configPath=os.path.split(os.path.split(path)[0])[0]
    configName='user'+'.conf'
    content='''
[user]
userName=Unknown
siteName=Unknown


'''
class pageConfig(config):
    _dayNum=systemConfig().getConfig('system','daynum')
    configPath=systemConfig().getConfig('system',_dayNum)
    configName='page'+'.conf'
    content='''
[page]

'''
class seriesConfig(config):
    configPath=systemConfig().getConfig('system','series')
    print configPath
    configName='series'+'.conf'
    content='''
[series]

'''

if __name__=='__main__':
    systemConfig().setConfig('system','fuckyout','tom')



#coding:utf-8
import os
import sys
reload(sys)
sys.setdefaultencoding('utf8')
sys.path.append('../')
from config import config
class theme():
    def __init__(self):
        systemConfig=config.systemConfig()
        themePath=systemConfig.getConfig('system','theme')
        allTheme= os.listdir(themePath)
        m=''
        index=''
        other=''
        for item in allTheme:
            if os.path.isfile(themePath+'/'+item):
                if item.find('.')>=0:
                    name=item.split('.')[0]
                    if name[0]=="m":
                        m=m+' '+name
                    elif name[0]=="i":
                        index=index+' '+name
                    else:
                        other=other+' '+name+' '

        systemConfig.setConfig('theme','m',m)
        systemConfig.setConfig('theme','index',index)
        systemConfig.setConfig('theme','other',other)
    def useTheme(self,name):
        systemConfig=config.systemConfig()
        themePath=systemConfig.getConfig('system','theme')
        try:
            with open(themePath+'/'+name+'.html','r')as f:
                content=f.read()
        except IOError:
            print 'no %s'%(name)
            return ''
        return content



                

                






#coding:utf-8
import os
import sys
reload(sys)
sys.setdefaultencoding('utf8')
sys.path.append('../')
from relation import relation
from config import config
import markdown
import re
from theme import theme
def cutContent(opcontent,number):
    content=opcontent
    content.reverse()
    warn='Conversion Failed:\nYou may have DESTORYED the config words when you editing!!!\nYou may edit your article bellow the config words\n'
    try:
        a=content.index('notes:\n')
        b=content.index('compose on the top of these words plz\n')
        c=content.index('please write title on the first line\n')
        d=content.index('you can change the choices but do not destroy these words!!!\n')
        if d==(c-1):
            if c==(b-1):
                if b==(a-1):
                    pass
                else:
                    raise UserWarning(warn)
            else:
                raise UserWarning(warn)
        else:
            raise UserWarning(warn)
    except ValueError:
        print warn
        sys.exit()
    except UserWarning:
        print warn
        sys.exit()
    configContent=content[0:a+1]
    configContent.reverse()
    articleContent=content[a+1::]
    isEmpty=True
    for item in articleContent:
        item=item.strip()
        if item =='':
            pass
        elif item=='\n':
            pass
        else:
            isEmpty=False
    if isEmpty:
        print '未编辑，已退出'
        if os.path.exists(config.pageConfig().getConfig(number,'path')):
            os.remove(config.pageConfig().getConfig(number,'path'))
            config.pageConfig().deleteSection(number)
        relation()
        sys.exit()


    articleContent.reverse()
    title=articleContent[0]
    del articleContent[0]
    return title,articleContent,configContent

def readConfigContent(content,title,number):
    pageConfig=config.pageConfig()
    pageConfig.setConfig(number,'title',title)
    for item in content:
        if item.find('=')>=0:
            key=item.split('=')[0]
            value=item.split('=')[1]
            pageConfig.setConfig(number,key,value)
        else:
            pass
def initHtml(html,themeName):
    themePath=config.systemConfig().getConfig('system','sources')+'/'+themeName+'/'
    for item in os.listdir(themePath):
        sourcesPattern="href\={1}\".*"+item+"\""
        html=re.sub(sourcesPattern,"href=\"./../../../../sources/"+themeName+'/'+item+"\"",html)
    for item in os.listdir(themePath):
        sourcesPattern="src\={1}\".*"+item+"\""
        html=re.sub(sourcesPattern,"src=\"./../../../../sources/"+themeName+'/'+item+"\"",html)
    return html
def replace(articleHtml,number):
    pageConfig=config.pageConfig()
    systemConfig=config.systemConfig()
    userConfig=config.userConfig()
    theme1=pageConfig.getConfig(number,'theme')
    m=systemConfig.getConfig('theme','m').split(' ')
    if not theme1 in m:
        print 'No theme %s'%(theme1)
        print 'Please Check'
        sys.exit()
    html=theme().useTheme(theme1)
    html=initHtml(html,theme1)
    if html=='':
        print "Didn't find the theme" 
        sys.exit()
    htmlPath=systemConfig.getConfig('system',systemConfig.getConfig('system','dayNum'))
    htmlPath=htmlPath[::-1]
    htmlPath=htmlPath.replace('dm','lmth',1)
    htmlPath=htmlPath[::-1]+'/'+number+'.html'
    a=htmlPath
    htmlPath=htmlPath[htmlPath.index('html')::]
    htmlPath='./../../../../'+htmlPath
    pageConfig.setConfig(number,'htmlLink',a)
    pageConfig.setConfig(number,'articleLink',htmlPath)
    avatar=userConfig.getConfig('user','avatar')
    if userConfig.getConfig('user','avatar').find('http://')<0:
        if userConfig.getConfig('user','avatar').find('https://')<0:
            avatar='./../../../../'+userConfig.getConfig('user','avatar')
    navigation=''
    try:
        for item,navLinks in systemConfig.items('navigation'):
            navigation=navigation+'<a href="%s">%s</a>'%(navLinks,item)
    except: 
        pass
    change=(('<!---article---><!---article--->',articleHtml),
            ('<!---articletitle---><!---articletitle--->',pageConfig.getConfig(number,'title')),
            ('<!---username---><!---username--->',userConfig.getConfig('user','userName')),
            ('<!---sitename---><!---sitename--->',userConfig.getConfig('user','userName')),
            ('<!---articledate---><!---articledate--->',systemConfig.getConfig('system','time')),
            ('<!---userquote---><!---userquote--->',userConfig.getConfig('user','quote')),
            ('<!---copyright---><!---copyright--->',userConfig.getConfig('user','copyright')),
            ('<!---useravatar---><!---useravatar--->',avatar),
            ('<!---ariclelink---><!---ariclelink--->',pageConfig.getConfig(number,'articleLink')),
            ('<!---email---><!---email--->','mailto:'+userConfig.getConfig('user','email')),
            ('<!---github---><!---github--->',userConfig.getConfig('user','github')),
            ('<!---sitelink---><!---sitelink--->','./'),
            ('</head>',systemConfig.getConfig('system','headScript')+'</head>'),
            ('<!---nav---><!---nav--->',navigation),
    )
    
    for pattern,subs in change:
        html=re.sub(pattern,subs,html)

    return html,a

def writeIn(html,path):
    with open(path,'w')as f:
        f.write(html)




def m2h(content):
    exts=['markdown.extensions.extra','markdown.extensions.codehilite','markdown.extensions.tables','markdown.extensions.toc']
    articleHtml=markdown.markdown(content,extensions=exts)
    return articleHtml
def handleArticleContent(content,number):
    content=''.join(content)#把ｌｉｓｔ变为ｓｔｒ
    articleHtml=m2h(content)
    html,htmlPath=replace(articleHtml,number)
    writeIn(html,htmlPath)
    
    



class md2html():
    def __init__(self,mdPath):
        number=os.path.split(mdPath)[1].split('.')[0]
        with open(mdPath,'r')as f:
            self.content=f.readlines()
        title,articleContent,configContent=cutContent(self.content,number)
        readConfigContent(configContent,title,number)
        handleArticleContent(articleContent,number)




#coding:utf-8
#检查目录的完整性
##获取系统信息，更新系统配置文件
import os
import platform
import sys
import time
import shutil
reload(sys)
sys.setdefaultencoding('utf8')
sys.path.append('../')
from config import config
from theme import theme
def mkdir(tree):
    systemConfig=config.systemConfig()
    systemPath=systemConfig.getConfig('system','systemPath')
    oldFather='systemPath'
    fatherPath=systemPath
    for (father,son) in tree:
        if not oldFather==father:
            oldFather=father
            fatherPath=systemConfig.getConfig('system',father)
        if not os.path.exists(fatherPath+'/'+son):
            os.mkdir(fatherPath+'/'+son)
        systemConfig.setConfig('system',son,fatherPath+'/'+son)
def moveThemeSources():
    systemConfig=config.systemConfig()
    themePath=systemConfig.getConfig('system','theme')
    for item in os.listdir(themePath):
        if not os.path.isfile(themePath+'/'+item):
            if os.path.exists(systemConfig.getConfig('system','sources')+'/'+item):
                shutil.rmtree(systemConfig.getConfig('system','sources')+'/'+item)
                shutil.copytree(themePath+'/'+item,systemConfig.getConfig('system','sources')+'/'+item)
            else:
                shutil.copytree(themePath+'/'+item,systemConfig.getConfig('system','sources')+'/'+item)
def howManyPost():
    systemConfig=config.systemConfig()
    dayNum=systemConfig.getConfig('system','daynum')
    dayPath=systemConfig.getConfig('system',dayNum)
    systemConfig.setConfig('system','post',len(os.listdir(dayPath)))

    

class check():
    path=os.path.split(os.path.abspath(__file__))[0]
    def __init__(self):
        systemConfig=config.systemConfig()
        systemInfo={'systemPath':os.path.split(os.path.split(self.path)[0])[0],
                'os':platform.system(),
                'pythonVersion':platform.python_version(),
                'pythonPath':sys.executable,
                'configPath':systemConfig.configPath,
                'yearNum':time.strftime('%Y'),
                'monNum':time.strftime('%m'),
                'dayNum':time.strftime('%d'),
                'time':time.strftime('%B %d, %Y')

                }
        for item in systemInfo:
            systemConfig.setConfig('system',item,systemInfo[item])
        yearNum=systemConfig.getConfig('system','yearNum')
        monNum=systemConfig.getConfig('system','monNum')
        dayNum=systemConfig.getConfig('system','dayNum')



        #检查并创建目录，复制主题
        tree=(('systemPath','bin'),('systemPath','theme'),
                ('bin','html'),('bin','md'),('bin','series'),('bin','sources'),
                ('html',yearNum),(yearNum,monNum),(monNum,dayNum),
                ('md',yearNum),(yearNum,monNum),(monNum,dayNum)
              ) 
        if systemConfig.isExistOption('system',str(int(yearNum)-1)):
            systemConfig.deleteOption('system',str(int(yearNum)-1))
        mkdir(tree)   
        moveThemeSources()
        config.pageConfig()
        config.userConfig()
        config.seriesConfig()
        theme()
        howManyPost()



if __name__=='__main__':
    check()



#coding:utf-8

#调用这个类，得到内容属性
import os
import sys
reload(sys)
sys.setdefaultencoding('utf8')
sys.path.append('../')
from config import config
from theme import theme
def vim(path):
    try:
        os.system('vim '+path)
    except:
        print "Error: Didn't find editor Vim on your computer\nYou maight haven't install it or set right system path\nIn this program, Vim is the default Editor, if you don't like it, you can change the sourde code in %s and I like to help"%(os.path.abspath(__file__))
        sys.exit()


def preContent(number):
    systemConfig=config.systemConfig()
    try:
        systemConfig.addSection('optional')
    except:
        pass
    contents=('series','theme','m','indexTheme','index')
    instead=[]
    for item in contents:
        if item in ('m','index'):
            instead.append(systemConfig.getConfig('theme',item))
        else:
            instead.append(systemConfig.getConfig('optional',item))
    instead=tuple(instead)
    content='''












notes:
compose on the top of these words plz
please write title on the first line
you can change the choices but do not destroy these words!!!
series=%s

theme=%s
choices of theme: %s
indexTheme=%s
choices of indexTheme: %s
'''
    content=content%instead

    return content
def creatMd(path,number):
    systemConfig=config.systemConfig()
    pageConfig=config.pageConfig()
    if not os.path.exists(path+'/'+str(number)+'.md'):
        with open(path+'/'+str(number)+'.md','w')as f:
            f.write(preContent(str(number)))
    try:
        pageConfig.addSection(str(number))
    except:
        pass
    pageConfig.setConfig(str(number),'path',path+'/'+str(number)+'.md')
    vim(path+'/'+str(number)+'.md')
    return path+'/'+str(number)+'.md'

class edit():
    def __init__(self,number=0):
        systemConfig=config.systemConfig()
        dayNum=systemConfig.getConfig('system','daynum')
        dayPath=systemConfig.getConfig('system',dayNum)
        if number==1:
            number=systemConfig.getConfig('system','post')
            number=int(number)-1
            if number==-1:
                number=0
            number=str(number)
        self.mdPath=creatMd(dayPath,number)





#coding:utf-8
import sys
import os
import re
reload(sys)
sys.setdefaultencoding('utf8')
sys.path.append('../')
from config import config
import shutil
def changeMdtoHtml(pageConfigPath):
    pageHtmlPath=pageConfigPath[::-1]
    pageHtmlPath=pageHtmlPath.replace('dm','lmth',1)
    pageHtmlPath=pageHtmlPath[::-1]
    return pageHtmlPath
def replace(change,content):
    for pattern,subs in change:
        content=re.sub(pattern,subs,content)
    return content
def makeDayRelation(pageConfig,mds):
    m=0
    for item in mds:
        if item==mds[0]:
            pass
        if item==mds[-1]:
            try:
                if len(mds)<1:
                    with open(pageConfig.getConfig(str(mds[m]),'htmllink'),'r')as f:
                        content=f.read()
                    articlelink=pageConfig.getConfig(str(mds[m]),'articlelink')
                    change=(('href=.*<!---next--->.*<!---next--->',"href=\""+articlelink+"#<!---next---><!---next--->"),
                            ('src=.*<!---next--->.*<!---next--->',"src=\""+articlelink+"#<!---next---><!---next--->"))
                    content=replace(change,content)
                else:
                    with open(pageConfig.getConfig(str(mds[m]),'htmllink'),'r')as f:
                        content=f.read()
                    articlelink=pageConfig.getConfig(str(mds[m]),'articlelink')
                    preArticleLink=pageConfig.getConfig(str(mds[m-1]),'articlelink')
                    print preArticleLink
                    change=(('href=.*<!---next--->.*<!---next--->',"href=\""+articlelink+"#<!---next---><!---next--->"),
                            ('src=.*<!---next--->.*<!---next--->',"src=\""+articlelink+"#<!---next---><!---next--->"),
                            ('href=.*<!---previous--->.*<!---previous--->',"href=\""+preArticleLink+"#<!---previous---><!---previous--->"))
                    content=replace(change,content)
                    
                    with open(pageConfig.getConfig(str(mds[m-1]),'htmllink'),'r')as f:
                        pContent=f.read()
                    change=(('href=.*<!---next--->.*<!---next--->',"href=\""+articlelink+"#<!---next---><!---next--->"),
                            ('src=.*<!---next--->.*<!---next--->',"src=\""+articlelink+"#<!---next---><!---next--->"))
                    pContent=replace(change,pcontent)
                    with open(pageConfig.getConfig(str(mds[m-1]),'htmllink'),'w') as f:
                        f.write(pContent)

                

            except:
                pass
        else:
            try:
                with open(pageConfig.getConfig(str(mds[m]),'htmllink'),'r')as f:
                    content=f.read()
                
                change=(('href=.*<!---previous--->.*<!---previous--->', "href=\""+ pageConfig.getConfig(str(mds[m-1]),'articlelink') +'#<!---previous---><!---previous--->' ),
                        ('href=.*<!---next--->.*<!---next--->', "href=\""+pageConfig.getConfig(str(mds[m+1]),'articlelink')+'#<!---next---><!---next--->')
                        )
                content=replace(change,content)
                
               
            except:
                pass
        try:
            with open(pageConfig.getConfig(str(mds[m]),'htmllink'),'w')as fp:
                    fp.write(content)
        except:
            pass
                
        m=m+1
def noName(path):
    path=path.split('/')
    path.reverse()
    path=path[0:5]
    path.reverse()
    path='./../../../../'+('/').join(path)
    return path



def makeDayDayRelation(pageConfig,systemConfig):
    monNum=systemConfig.getConfig('system','monnum')
    path=systemConfig.getConfig('system',monNum)
    path=changeMdtoHtml(path)
    dates=os.listdir(path)
    intDate=[]
    for item in dates:
        intDate.append(int(item))
    intDate.sort()
    if len(dates)<=0:
        pass
    else:
        m=0
        for item in  intDate:
            if item==intDate[0]:
                pass
            else:
                #找到今天的第一片
                #找到前一天的最后一片
                #前一天的最后一片，改变它的next
                #今天的第一篇，改变它的previous
                #日期加一
                todayPath=path+'/'+str(intDate[m])
                n=[]
                for item in os.listdir(todayPath):
                    n.append(int(item.split('.')[0]))
                n.sort()
                firstInToday=todayPath+'/'+str(n[0])+'.html'
                FarticlePath=noName(firstInToday)




                yestodayPath=path+'/'+str(intDate[m-1])
                n=[]
                for item in os.listdir(yestodayPath):
                    n.append(int(item.split('.')[0]))
                n.sort()
                lastInYestoday=yestodayPath+'/'+str(n[-1])+'.html'
                LarticlePath=noName(lastInYestoday)
                with open(lastInYestoday,'r') as f:
                    lastContent=f.read()
                change=(('href=.*<!---next--->.*<!---next--->',"href=\""+FarticlePath+'#<!---next---><!---next--->'),
                        ('href=.*<!---next--->.*<!---next--->',"href=\""+FarticlePath+'#<!---next---><!---next--->')
                        )
                lastContent=replace(change,lastContent)
                with open(lastInYestoday,'w')as f:
                    f.write(lastContent)

                with open(firstInToday,'r')as f:
                    firstContent=f.read()
                change=(('href=.*<!---previous--->.*<!---previous--->',"href=\""+LarticlePath+'#<!---previous---><!---previous--->'),
                        ('href=.*<!---previous--->.*<!---previous--->',"href=\""+LarticlePath+'#<!---previous---><!---previous--->')
                        )
                firstContent=replace(change,firstContent)
                with open(firstInToday,'w')as f:
                    f.write(firstContent)



            m=m+1


            




def makeMonthRelation(systemConfig,mds):
    yearNum=systemConfig.getConfig('system','yearnum')
    yearPath=systemConfig.getConfig('system',yearNum)
    yearPath=noName(yearPath)
    months=os.listdir(yearPath)
    intMonths=[]
    for item in months:
        intMonths.append(int(item))
    intMonths.sort()
    #未完待续





class relation():
    def __init__(self):
        systemConfig=config.systemConfig()
        pageConfig=config.pageConfig()
        pageConfigPath=pageConfig.configPath
        pageHtmlPath=changeMdtoHtml(pageConfigPath)
        if len(os.listdir(pageConfigPath))<=1:
            shutil.rmtree(pageConfigPath)
            shutil.rmtree(pageHtmlPath)
            sys.exit()
        mds=[]
        for item in os.listdir(pageConfigPath):
            try:
                mds.append(int(item.split('.')[0]))
            except:
                pass
        mds.sort()
        makeDayRelation(pageConfig,mds)
        makeDayDayRelation(pageConfig,systemConfig)
        makeMonthRelation(systemConfig,mds)






        



#coding:utf-8
import sys
class inputPara():
    dicts={'g':'gui','gui':'gui','GUI':'gui','Gui':'gui','gUi':'gui','-g':'gui',
            'h':'help','-h':'help','help':'help',
            'qiniu':'qiniu','qn':'qiniu','Qiniu':'qiniu','QINIU':'qiniu',
            'th':'theme','Theme':'theme','theme':'theme',
            'siteInfo':'siteInfo','siteinfo':'siteInfo','si':'siteInfo',
            'index':'index',
            'new':'new'
            }

    
    def whatCommand(self):
        a=''
        for command in sys.argv[1:]:
            try:
                a=a+self.dicts[command]+' '
            except:
                a=a+'0 '
        a=a.split()
        if '0'in a:
            print "Error: Unkown command\nplease enter the true parameters\nfor more information,please use -h "
            a=['error']
            sys.exit()
        elif 'help'in a:      
            print "g  run the GUI\nh  print this help message\nqiniu  update the qiniu Config File\nsiteInfo  update the site information"
            sys.exit()
        return a
if __name__=='__main__':
    thein=inputPara()
    print thein.whatCommand()




# coding:utf-8
from setuptools import setup, find_packages

setup(
        name='sarita',
        version='1.0',
        description=('create your site easily'),
        long_description=open('README.rst').read(),
        author='Mr.Lution',
        author_email='mrlution@qq.com',
        maintainer='Mr.Lution',
        maintainer_email='mrlution@qq.com',
        install_requires=[],
        license='GPL License',
        packages=find_packages(),
        include_package_data=True,
        zip_safe=False,
        keywords='sarita',
        platforms=['Linux'],
        url='https://uestcman.github.io',
        classifiers=[
            'Programming Language :: Python :: 2.7',
            ],

        )
